Imports System.IO
Imports Microsoft.Win32

Public Class Form1
    ' API declarations.
    Private Const SPI_SETDESKWALLPAPER As Integer = &H14
    Private Const SPIF_UPDATEINIFILE As Integer = &H1
    Private Const SPIF_SENDWININICHANGE As Integer = &H2
    Private Declare Auto Function SystemParametersInfo Lib "user32.dll" ( _
        ByVal uAction As Integer, ByVal uParam As Integer, _
        ByVal lpvParam As String, ByVal fuWinIni As Integer) As Integer

    ' SaveSetting/GetSetting values.
    Private Const APP_NAME = "Backer"
    Private Const SECTION_NAME = "Settings"

    ' Settings.
    Private m_UpdateRegistry As Boolean
    Private m_DisplayStyle As String

    ' The list of files we will pick from.
    Private m_FileNames As Collection

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        ' Restore the previous form position and state.
        RestorePosition(Me, APP_NAME)

        Me.Enabled = False
        Me.Refresh()

        ' Load saved settings.
        LoadConfig()

        ' Give the small buttons transparent backgrounds.
        MakeButtonTransparent(btnAddDirectory)
        MakeButtonTransparent(btnRemoveDirectory)
        MakeButtonTransparent(btnMoveFile)
        MakeButtonTransparent(btnDeleteFile)

        ' Apply the current settings. (This also disables the button.)
        ApplyChanges()

        Me.Enabled = True
    End Sub

    ' Give the button a transparent background.
    Private Sub MakeButtonTransparent(ByVal btn As Button)
        Dim bm As Bitmap = btn.Image
        bm.MakeTransparent(bm.GetPixel(0, 0))
    End Sub

    ' See if there are unsaved changes.
    Private Sub Form1_FormClosing(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosingEventArgs) Handles Me.FormClosing
        If btnApplyChanges.Enabled Then
            Select Case MessageBox.Show("Do you want to save your changes?", _
                    "Save Changes?", _
                    MessageBoxButtons.YesNoCancel, _
                    MessageBoxIcon.Question)
                Case Windows.Forms.DialogResult.Yes
                    ' Apply the changes.
                    ApplyChanges()

                Case Windows.Forms.DialogResult.No
                    ' Don't apply the changes.

                Case Windows.Forms.DialogResult.Cancel
                    ' Cancel the close.
                    e.Cancel = True
            End Select
        End If

        ' Save the form position and state.
        If Not e.Cancel Then
            SavePosition(Me, APP_NAME)
        End If
    End Sub

    ' Load previously saved settings.
    Public Sub LoadConfig()
        Dim pause As Integer = GetSetting(APP_NAME, SECTION_NAME, "Interval", "300")
        txtNumSeconds.Text = pause.ToString()
        tmrNewPicture.Interval = pause * 1000

        m_UpdateRegistry = CBool(GetSetting(APP_NAME, SECTION_NAME, "UpdateRegistry", False))
        chkUpdateRegistry.Checked = m_UpdateRegistry

        m_DisplayStyle = GetSetting(APP_NAME, SECTION_NAME, "Style", "Centered")
        cboStyle.SelectedItem = m_DisplayStyle

        ' Load the directories.
        Dim i As Integer = 0
        Do
            Dim new_dir As String = GetSetting(APP_NAME, SECTION_NAME, "Directory" & i.ToString(), "")
            If new_dir.Length = 0 Then Exit Do
            If My.Computer.FileSystem.DirectoryExists(new_dir) Then
                lstDirectories.Items.Add(new_dir)
            End If
            i += 1
        Loop
    End Sub

    ' Save the program's configuration.
    Public Sub SaveConfig()
        ' Delete old settings.
        Try
            DeleteSetting(APP_NAME, SECTION_NAME)
        Catch ex As Exception
        End Try

        ' Save the current settings
        Dim pause As Integer = tmrNewPicture.Interval \ 1000
        SaveSetting(APP_NAME, SECTION_NAME, "Interval", pause.ToString())
        SaveSetting(APP_NAME, SECTION_NAME, "UpdateRegistry", m_UpdateRegistry)
        SaveSetting(APP_NAME, SECTION_NAME, "Style", m_DisplayStyle)

        ' Save the directories.
        For i As Integer = 0 To lstDirectories.Items.Count - 1
            SaveSetting(APP_NAME, SECTION_NAME, "Directory" & i.ToString(), lstDirectories.Items(i))
        Next i
    End Sub

    ' Remove the selected directory.
    Private Sub btnRemoveDirectory_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnRemoveDirectory.Click
        If lstDirectories.SelectedIndex < 0 Then
            btnRemoveDirectory.Enabled = False
            Exit Sub
        End If
        lstDirectories.Items.RemoveAt(lstDirectories.SelectedIndex)
        btnApplyChanges.Enabled = True
    End Sub

    ' Let the user add a directory.
    Private Sub btnAddDirectory_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnAddDirectory.Click
        If fbdDirectory.ShowDialog() = Windows.Forms.DialogResult.OK Then
            lstDirectories.Items.Add(fbdDirectory.SelectedPath)
            btnApplyChanges.Enabled = True
        End If
    End Sub

    ' Apply the values in the controls.
    Private Sub btnApplyChanges_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnApplyChanges.Click
        ApplyChanges()
    End Sub
    Private Sub ApplyChanges()
        ' Get the new interval.
        Dim pause As Integer = Val(txtNumSeconds.Text)
        If pause < 1 Then
            pause = 1
            txtNumSeconds.Text = pause.ToString()
        End If

        ' Reset the timer.
        tmrNewPicture.Interval = pause * 1000
        tmrNewPicture.Enabled = False
        tmrNewPicture.Enabled = True

        ' Get the new update flag.
        m_UpdateRegistry = chkUpdateRegistry.Checked

        ' Get the new display style.
        m_DisplayStyle = cboStyle.Text

        ' Load the list of files.
        LoadFileList()

        ' Save the settings.
        SaveConfig()

        ' Disable the Apply button again.
        btnApplyChanges.Enabled = False
    End Sub

    ' Load the file list.
    Private Sub LoadFileList()
        m_FileNames = New Collection
        Dim dir_names As New Collection
        For i As Integer = 0 To lstDirectories.Items.Count - 1
            ' Make sure the directory exists.
            Dim dir_name As String = lstDirectories.Items(i)
            If My.Computer.FileSystem.DirectoryExists(dir_name) Then
                dir_names.Add(dir_name)

                ' Get the files in the directory.
                Dim dir_info As New System.IO.DirectoryInfo(dir_name)
                Dim files() As FileInfo = _
                    dir_info.GetFiles("*.*", SearchOption.TopDirectoryOnly)
                For Each file_info As FileInfo In files
                    Select Case file_info.Extension.ToLower()
                        Case ".bmp", ".jpg", ".jpeg", ".gif", ".png"
                            m_FileNames.Add(file_info.FullName)
                    End Select
                Next file_info
            End If
        Next i
        lblNumFiles.Text = m_FileNames.Count.ToString() & " files"

        ' Rebuild the directory list.
        lstDirectories.Items.Clear()
        For i As Integer = 1 To dir_names.Count
            lstDirectories.Items.Add(dir_names(i))
        Next i
        lstDirectories.SelectedIndex = -1
        lstDirectories_SelectedIndexChanged(Nothing, Nothing)

        ' Display a picture.
        tmrNewPicture.Enabled = False
        tmrNewPicture.Enabled = True
        tmrNewPicture_Tick(Nothing, Nothing)
    End Sub

    ' The user changed something. Enable the Apply button.
    Private Sub Value_Changed(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles chkUpdateRegistry.CheckedChanged, txtNumSeconds.TextChanged, cboStyle.SelectedIndexChanged
        btnApplyChanges.Enabled = True
    End Sub

    ' Display the next picture.
    Private Sub tmrNewPicture_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles tmrNewPicture.Tick
        If m_FileNames.Count < 1 Then Exit Sub

        ' Pick a file.
        Dim rand As New Random()
        Dim i As Integer = rand.Next(1, m_FileNames.Count + 1)
        Dim file_name As String = m_FileNames.Item(i)

        ' Display it.
        DisplayPicture(file_name)
    End Sub

    ' Display the file on the desktop.
    Private Sub DisplayPicture(ByVal file_name As String)
        txtCurrentFile.Text = file_name

        Try
            ' Set the proper display style.
            Dim key As RegistryKey = Registry.CurrentUser.OpenSubKey("Control Panel\Desktop", True)
            Const STYLE_CENTERED As String = "0"
            Const STYLE_TILED As String = "1"
            Const STYLE_STRETCHED As String = "2"
            Const TILE_NO As String = "0"
            Const TILE_YES As String = "1"
            Select Case m_DisplayStyle
                Case "Stretched"
                    key.SetValue("TileWallpaper", TILE_NO)
                    key.SetValue("WallpaperStyle", STYLE_STRETCHED)
                Case "Tiled"
                    key.SetValue("TileWallpaper", TILE_YES)
                    key.SetValue("WallpaperStyle", STYLE_TILED)
                Case "Centered"
                    key.SetValue("TileWallpaper", TILE_NO)
                    key.SetValue("WallpaperStyle", STYLE_CENTERED)
            End Select

            ' Load the file.
            Dim bm1 As New Bitmap(file_name)
            Dim bm As New Bitmap(bm1)
            bm1.Dispose()

            ' Save the image into a temporary bitmap file.
            Dim temp_dir As String = System.IO.Path.GetTempPath()
            Dim temp_file As String = temp_dir & "backer_temp.bmp"
            bm.Save(temp_file, System.Drawing.Imaging.ImageFormat.Bmp)

            ' Set the desktop background to this file.
            If m_UpdateRegistry Then
                SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, temp_file, SPIF_UPDATEINIFILE Or SPIF_SENDWININICHANGE)
            Else
                SystemParametersInfo(SPI_SETDESKWALLPAPER, 0, temp_file, 0)
            End If

        Catch ex As Exception
            MessageBox.Show("Error displaying picture " & file_name & "." & vbCrLf & ex.Message, _
                "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    ' Display another picture.
    Private Sub btnNextPicture_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnNextPicture.Click
        Me.Cursor = Cursors.WaitCursor
        Me.Refresh()

        tmrNewPicture_Tick(Nothing, Nothing)

        Me.Cursor = Cursors.Default
    End Sub

    ' Load this picture again. (In case it was modified.)
    Private Sub btnReloadPicture_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnReloadPicture.Click
        Me.Cursor = Cursors.WaitCursor
        Me.Refresh()

        DisplayPicture(txtCurrentFile.Text)

        Me.Cursor = Cursors.Default
    End Sub

    ' Edit the current picture.
    Private Sub btnEditPicture_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnEditPicture.Click
        System.Diagnostics.Process.Start(txtCurrentFile.Text)

        ' Reset the timer.
        tmrNewPicture.Enabled = False
        tmrNewPicture.Enabled = True
    End Sub

    ' Save the form's size and position.
    Private Sub SavePosition(ByVal frm As Form, ByVal app_name As String)
        SaveSetting(app_name, "Geometry", "WindowState", frm.WindowState)
        If frm.WindowState = FormWindowState.Normal Then
            SaveSetting(app_name, "Geometry", "Left", frm.Left)
            SaveSetting(app_name, "Geometry", "Top", frm.Top)
            SaveSetting(app_name, "Geometry", "Width", frm.Width)
            SaveSetting(app_name, "Geometry", "Height", frm.Height)
        Else
            SaveSetting(app_name, "Geometry", "Left", frm.RestoreBounds.Left)
            SaveSetting(app_name, "Geometry", "Top", frm.RestoreBounds.Top)
            SaveSetting(app_name, "Geometry", "Width", frm.RestoreBounds.Width)
            SaveSetting(app_name, "Geometry", "Height", frm.RestoreBounds.Height)
        End If
    End Sub

    ' Restore the form's size and position.
    Private Sub RestorePosition(ByVal frm As Form, ByVal app_name As String)
        frm.SetBounds( _
            GetSetting(app_name, "Geometry", "Left", Me.RestoreBounds.Left), _
            GetSetting(app_name, "Geometry", "Top", Me.RestoreBounds.Top), _
            GetSetting(app_name, "Geometry", "Width", Me.RestoreBounds.Width), _
            GetSetting(app_name, "Geometry", "Height", Me.RestoreBounds.Height) _
        )
        frm.WindowState = GetSetting(app_name, "Geometry", "WindowState", Me.WindowState)
    End Sub

    ' Enable or disable the remove directory button as appropriate.
    Private Sub lstDirectories_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lstDirectories.SelectedIndexChanged
        btnRemoveDirectory.Enabled = (lstDirectories.SelectedIndex >= 0)
    End Sub

    ' Move the current file.
    Private Sub btnMoveFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnMoveFile.Click
        Dim file_name As String = txtCurrentFile.Text
        Try
            ' Let the user pick a new file name.
            Dim file_info As New FileInfo(file_name)
            sfdPicture.FileName = file_info.Name
            If sfdPicture.ShowDialog() = Windows.Forms.DialogResult.OK Then
                My.Computer.FileSystem.MoveFile(file_name, sfdPicture.FileName)

                ' Reload the file list.
                LoadFileList()
            End If
        Catch ex As Exception
            MessageBox.Show("Error moving picture file " & file_name & "." & vbCrLf & ex.Message, _
                "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub

    ' Delete the current file.
    Private Sub btnDeleteFile_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnDeleteFile.Click
        Dim file_name As String = txtCurrentFile.Text
        Try
            My.Computer.FileSystem.DeleteFile(file_name, _
                FileIO.UIOption.AllDialogs, FileIO.RecycleOption.SendToRecycleBin)

            ' If we deleted the file, reload the file list.
            LoadFileList()

        Catch ex As OperationCanceledException
        Catch ex As Exception
            MessageBox.Show("Error deleting picture file " & file_name & "." & vbCrLf & ex.Message, _
                "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        End Try
    End Sub
End Class
